@@ -6,13 +6,20 @@
 #include "pysideproperty_p.h"
 #include "pysidesignal.h"
 #include "pysidesignal_p.h"
+#include "signalmanager.h"
 
-#include <shiboken.h>
+#include <autodecref.h>
 #include <pep384ext.h>
+#include <sbkconverter.h>
+#include <sbkstaticstrings.h>
+#include <sbkstring.h>
+#include <sbktypefactory.h>
 #include <signature.h>
 
 using namespace Shiboken;
 
+using namespace Qt::StringLiterals;
+
 extern "C"
 {
 
@@ -39,6 +46,7 @@ static PyObject *qProperty_fdel(PyObject *, void *);
 
 static PyMethodDef PySidePropertyMethods[] = {
     {"getter", reinterpret_cast<PyCFunction>(qPropertyGetter), METH_O, nullptr},
+    // "name@setter" handling
     {"setter", reinterpret_cast<PyCFunction>(qPropertySetter), METH_O,  nullptr},
     {"resetter", reinterpret_cast<PyCFunction>(qPropertyResetter), METH_O,  nullptr},
     {"deleter", reinterpret_cast<PyCFunction>(qPropertyDeleter), METH_O, nullptr},
@@ -94,12 +102,12 @@ PyTypeObject *PySideProperty_TypeF(void)
 PySidePropertyPrivate::PySidePropertyPrivate() noexcept = default;
 PySidePropertyPrivate::~PySidePropertyPrivate() = default;
 
-PyObject *PySidePropertyPrivate::getValue(PyObject *source)
+PyObject *PySidePropertyPrivate::getValue(PyObject *source) const
 {
     if (fget) {
         Shiboken::AutoDecRef args(PyTuple_New(1));
         Py_INCREF(source);
-        PyTuple_SET_ITEM(args, 0, source);
+        PyTuple_SetItem(args, 0, source);
         return  PyObject_CallObject(fget, args);
     }
     return nullptr;
@@ -107,18 +115,18 @@ PyObject *PySidePropertyPrivate::getValue(PyObject *source)
 
 int PySidePropertyPrivate::setValue(PyObject *source, PyObject *value)
 {
-    if (fset && value) {
+    if (fset != nullptr && fset != Py_None && value != nullptr) {
         Shiboken::AutoDecRef args(PyTuple_New(2));
-        PyTuple_SET_ITEM(args, 0, source);
-        PyTuple_SET_ITEM(args, 1, value);
+        PyTuple_SetItem(args, 0, source);
+        PyTuple_SetItem(args, 1, value);
         Py_INCREF(source);
         Py_INCREF(value);
         Shiboken::AutoDecRef result(PyObject_CallObject(fset, args));
         return (result.isNull() ? -1 : 0);
     }
-    if (fdel) {
+    if (fdel != nullptr && fdel != Py_None) {
         Shiboken::AutoDecRef args(PyTuple_New(1));
-        PyTuple_SET_ITEM(args, 0, source);
+        PyTuple_SetItem(args, 0, source);
         Py_INCREF(source);
         Shiboken::AutoDecRef result(PyObject_CallObject(fdel, args));
         return (result.isNull() ? -1 : 0);
@@ -129,10 +137,10 @@ int PySidePropertyPrivate::setValue(PyObject *source, PyObject *value)
 
 int PySidePropertyPrivate::reset(PyObject *source)
 {
-    if (freset) {
+    if (freset != nullptr && freset != Py_None) {
         Shiboken::AutoDecRef args(PyTuple_New(1));
         Py_INCREF(source);
-        PyTuple_SET_ITEM(args, 0, source);
+        PyTuple_SetItem(args, 0, source);
         Shiboken::AutoDecRef result(PyObject_CallObject(freset, args));
         return (result.isNull() ? -1 : 0);
     }
@@ -144,16 +152,20 @@ void PySidePropertyPrivate::metaCall(PyObject *source, QMetaObject::Call call, v
     switch (call) {
     case QMetaObject::ReadProperty: {
         AutoDecRef value(getValue(source));
-        auto *obValue = value.object();
-        if (obValue) {
-            Conversions::SpecificConverter converter(typeName);
-            if (converter) {
-                converter.toCpp(obValue, args[0]);
-            } else {
-                // PYSIDE-2160: Report an unknown type name to the caller `qtPropertyMetacall`.
-                PyErr_SetObject(PyExc_StopIteration, obValue);
-            }
+        if (value.isNull())
+            return;
+        if (typeName == "PyObject"_ba) {
+            // Manual conversion, see PyObjectWrapper converter registration
+            auto *pw = reinterpret_cast<PySide::PyObjectWrapper *>(args[0]);
+            pw->reset(value.object());
+            return;
         }
+        if (Conversions::SpecificConverter converter(typeName); converter) {
+            converter.toCpp(value.object(), args[0]);
+            return;
+        }
+        // PYSIDE-2160: Report an unknown type name to the caller `qtPropertyMetacall`.
+        PyErr_SetObject(PyExc_StopIteration, value.object());
     }
         break;
 
@@ -188,7 +200,7 @@ static PyObject *qpropertyTpNew(PyTypeObject *subtype, PyObject * /* args */, Py
 static int qpropertyTpInit(PyObject *self, PyObject *args, PyObject *kwds)
 {
     PyObject *type{};
-    auto data = reinterpret_cast<PySideProperty *>(self);
+    auto *data = reinterpret_cast<PySideProperty *>(self);
     PySidePropertyPrivate *pData = data->d;
 
     static const char *kwlist[] = {"type", "fget", "fset", "freset", "fdel", "doc", "notify",
@@ -228,7 +240,7 @@ static int qpropertyTpInit(PyObject *self, PyObject *args, PyObject *kwds)
     Py_XINCREF(pData->pyTypeObject);
     pData->typeName = PySide::Signal::getTypeName(type);
 
-    if (pData->typeName.isEmpty())
+    if (type == Py_None || pData->typeName.isEmpty())
         PyErr_SetString(PyExc_TypeError, "Invalid property type or type name.");
     else if (pData->constant && ((pData->fset && pData->fset != Py_None)
                                  || (pData->notify && pData->notify != Py_None)))
@@ -253,11 +265,9 @@ static int qpropertyTpInit(PyObject *self, PyObject *args, PyObject *kwds)
 static void qpropertyDeAlloc(PyObject *self)
 {
     qpropertyClear(self);
-    if (PepRuntime_38_flag) {
-        // PYSIDE-939: Handling references correctly.
-        // This was not needed before Python 3.8 (Python issue 35810)
-        Py_DECREF(Py_TYPE(self));
-    }
+    // PYSIDE-939: Handling references correctly.
+    // This was not needed before Python 3.8 (Python issue 35810)
+    Py_DECREF(Py_TYPE(self));
     PyObject_GC_UnTrack(self);
     PepExt_TypeCallFree(self);
 }
@@ -298,7 +308,7 @@ _property_copy(PyObject *old, PyObject *get, PyObject *set, PyObject *reset, PyO
     else
         doc = pData->doc;
 
-    auto notify = pData->notify ? pData->notify : Py_None;
+    auto *notify = pData->notify ? pData->notify : Py_None;
 
     PyObject *obNew = PyObject_CallFunction(type, const_cast<char *>("OOOOOsO" "bbb" "bbb"),
         pData->pyTypeObject, get, set, reset, del, doc.data(), notify,
@@ -338,39 +348,43 @@ static PyObject *qPropertyCall(PyObject *self, PyObject *args, PyObject * /* kw
 
 static PyObject *qProperty_fget(PyObject *self, void *)
 {
-    auto func = reinterpret_cast<PySideProperty *>(self)->d->fget;
-    auto ret = func != nullptr ? func : Py_None;
-    Py_INCREF(ret);
-    return ret;
+    auto *func = reinterpret_cast<PySideProperty *>(self)->d->fget;
+    if (func == nullptr)
+        Py_RETURN_NONE;
+    Py_INCREF(func);
+    return func;
 }
 
 static PyObject *qProperty_fset(PyObject *self, void *)
 {
-    auto func = reinterpret_cast<PySideProperty *>(self)->d->fset;
-    auto ret = func != nullptr ? func : Py_None;
-    Py_INCREF(ret);
-    return ret;
+    auto *func = reinterpret_cast<PySideProperty *>(self)->d->fset;
+    if (func == nullptr)
+        Py_RETURN_NONE;
+    Py_INCREF(func);
+    return func;
 }
 
 static PyObject *qProperty_freset(PyObject *self, void *)
 {
-    auto func = reinterpret_cast<PySideProperty *>(self)->d->freset;
-    auto ret = func != nullptr ? func : Py_None;
-    Py_INCREF(ret);
-    return ret;
+    auto *func = reinterpret_cast<PySideProperty *>(self)->d->freset;
+    if (func == nullptr)
+        Py_RETURN_NONE;
+    Py_INCREF(func);
+    return func;
 }
 
 static PyObject *qProperty_fdel(PyObject *self, void *)
 {
-    auto func = reinterpret_cast<PySideProperty *>(self)->d->fdel;
-    auto ret = func != nullptr ? func : Py_None;
-    Py_INCREF(ret);
-    return ret;
+    auto *func = reinterpret_cast<PySideProperty *>(self)->d->fdel;
+    if (func == nullptr)
+        Py_RETURN_NONE;
+    Py_INCREF(func);
+    return func;
 }
 
 static PyObject *qPropertyDocGet(PyObject *self, void *)
 {
-    auto data = reinterpret_cast<PySideProperty *>(self);
+    auto *data = reinterpret_cast<PySideProperty *>(self);
     PySidePropertyPrivate *pData = data->d;
 
     QByteArray doc(pData->doc);
@@ -389,9 +403,11 @@ static PyObject *qPropertyDocGet(PyObject *self, void *)
              * subclass instance instead, otherwise it gets shadowed by
              * __doc__ in the class's dict.
              */
-            auto get_doc_obj = get_doc.object();
-            int err = PyObject_SetAttr(self, PyMagicName::doc(), get_doc);
-            return err < 0 ? nullptr : (Py_INCREF(get_doc_obj), get_doc_obj);
+            auto *get_doc_obj = get_doc.object();
+            if (PyObject_SetAttr(self, PyMagicName::doc(), get_doc) < 0)
+                return nullptr;
+            Py_INCREF(get_doc_obj);
+            return get_doc_obj;
         }
         PyErr_Clear();
     }
@@ -400,7 +416,7 @@ static PyObject *qPropertyDocGet(PyObject *self, void *)
 
 static int qPropertyDocSet(PyObject *self, PyObject *value, void *)
 {
-    auto data = reinterpret_cast<PySideProperty *>(self);
+    auto *data = reinterpret_cast<PySideProperty *>(self);
     PySidePropertyPrivate *pData = data->d;
 
     if (String::check(value)) {
@@ -452,9 +468,9 @@ static PyObject *getFromType(PyTypeObject *type, PyObject *name)
     auto *attr = PyDict_GetItem(tpDict.object(), name);
     if (!attr) {
         PyObject *bases = type->tp_bases;
-        const Py_ssize_t size = PyTuple_GET_SIZE(bases);
+        const Py_ssize_t size = PyTuple_Size(bases);
         for (Py_ssize_t i = 0; i < size; ++i) {
-            PyObject *base = PyTuple_GET_ITEM(bases, i);
+            PyObject *base = PyTuple_GetItem(bases, i);
             attr = getFromType(reinterpret_cast<PyTypeObject *>(base), name);
             if (attr)
                 return attr;
@@ -466,24 +482,31 @@ static PyObject *getFromType(PyTypeObject *type, PyObject *name)
 namespace PySide::Property {
 
 static const char *Property_SignatureStrings[] = {
-    "PySide6.QtCore.Property(self,type:type,fget:typing.Callable=None,fset:typing.Callable=None,"
-        "freset:typing.Callable=None,fdel:typing.Callable=None,doc:str=None,"
-        "notify:typing.Callable=None,designable:bool=True,scriptable:bool=True,"
+    "PySide6.QtCore.Property(self,type:type,"
+        "fget:typing.Optional[collections.abc.Callable[[typing.Any],typing.Any]]=None,"
+        "fset:typing.Optional[collections.abc.Callable[[typing.Any,typing.Any],None]]=None,"
+        "freset:typing.Optional[collections.abc.Callable[[typing.Any,typing.Any],None]]=None,"
+        "doc:str=None,"
+        "notify:typing.Optional[PySide6.QtCore.Signal]=None,"
+        "designable:bool=True,scriptable:bool=True,"
         "stored:bool=True,user:bool=False,constant:bool=False,final:bool=False)",
-    "PySide6.QtCore.Property.deleter(self,fdel:typing.Callable)->PySide6.QtCore.Property",
-    "PySide6.QtCore.Property.getter(self,fget:typing.Callable)->PySide6.QtCore.Property",
-    "PySide6.QtCore.Property.read(self,fget:typing.Callable)->PySide6.QtCore.Property",
-    "PySide6.QtCore.Property.setter(self,fset:typing.Callable)->PySide6.QtCore.Property",
-    "PySide6.QtCore.Property.write(self,fset:typing.Callable)->PySide6.QtCore.Property",
+    "PySide6.QtCore.Property.deleter(self,fdel:collections.abc.Callable[[typing.Any],None])->PySide6.QtCore.Property",
+    "PySide6.QtCore.Property.getter(self,fget:collections.abc.Callable[[typing.Any],typing.Any])->PySide6.QtCore.Property",
+    "PySide6.QtCore.Property.read(self,fget:collections.abc.Callable[[typing.Any],typing.Any])->PySide6.QtCore.Property",
+    "PySide6.QtCore.Property.setter(self,fset:collections.abc.Callable[[typing.Any,typing.Any],None])->PySide6.QtCore.Property",
+    "PySide6.QtCore.Property.write(self,fset:collections.abc.Callable[[typing.Any,typing.Any],None])->PySide6.QtCore.Property",
+    "PySide6.QtCore.Property.__call__(self, func:collections.abc.Callable[...,typing.Any])->PySide6.QtCore.Property",
     nullptr}; // Sentinel
 
 void init(PyObject *module)
 {
-    if (InitSignatureStrings(PySideProperty_TypeF(), Property_SignatureStrings) < 0)
+    auto *propertyType = PySideProperty_TypeF();
+    if (InitSignatureStrings(propertyType, Property_SignatureStrings) < 0)
         return;
 
-    Py_INCREF(PySideProperty_TypeF());
-    PyModule_AddObject(module, "Property", reinterpret_cast<PyObject *>(PySideProperty_TypeF()));
+    auto *obPropertyType = reinterpret_cast<PyObject *>(propertyType);
+    Py_INCREF(obPropertyType);
+    PepModule_AddType(module, propertyType);
 }
 
 bool checkType(PyObject *pyObj)
@@ -537,12 +560,12 @@ bool isReadable(const PySideProperty * /* self */)
 
 bool isWritable(const PySideProperty *self)
 {
-    return self->d->fset != nullptr;
+    return self->d->fset != nullptr && self->d->fset != Py_None;
 }
 
 bool hasReset(const PySideProperty *self)
 {
-    return self->d->freset != nullptr;
+    return self->d->freset != nullptr && self->d->freset != Py_None;
 }
 
 bool isDesignable(const PySideProperty *self)
